import * as fs from 'fs';
import { badgeHashs } from './constants';
const readmeFile = badgeHashs.readmeFile;
const readmeTemplateFile = badgeHashs.readmeTemplateFile;
export default class Helper {
    constructor() {
        this.reportKeys = badgeHashs.hashes.jest;
        this.createReadme = (path = readmeTemplateFile) => {
            if (fs.existsSync(path)) {
                fs.copyFileSync(path, readmeFile);
                console.log('\x1b[1m\x1b[32m', `Template read succesfully. ${readmeFile} created!`);
                return true;
            }
            else {
                console.log('\x1b[1m\x1b[31m', `Error: ${readmeTemplateFile} file not not found.`);
                console.log('\x1b[1m\x1b[32m', `You must have a ${readmeTemplateFile} created. Please read the documentation.`);
                return false;
            }
        };
        this.insertBadges = (path = badgeHashs.jsonCoverageFile) => {
            let returnValue = false;
            const coveragePath = this.getCoveragePath(path);
            if (fs.existsSync(coveragePath)) {
                const coverageFile = fs.readFileSync(coveragePath, 'utf8');
                if (!coverageFile.length) {
                    console.log('\x1b[1m\x1b[31m', 'Malformed coverage report. Please run Jest again');
                    return false;
                }
                console.log('\x1b[1m\x1b[32m', `Reading coverage file from ${coveragePath}`);
                const report = JSON.parse(coverageFile);
                let readmeFileData = fs.readFileSync(readmeFile, 'utf8');
                this.reportKeys.forEach((reportKey) => {
                    const url = this.getBadge(report, reportKey.key);
                    if (url.length) {
                        const pattern = reportKey.value;
                        const startIndex = readmeFileData.indexOf(pattern);
                        const valueToChangeStart = readmeFileData.slice(startIndex + pattern.length + 1);
                        const valueToChangeIndex = valueToChangeStart.indexOf(')');
                        const valueToChangeFinal = valueToChangeStart.substring(0, valueToChangeIndex);
                        readmeFileData = readmeFileData.replace(valueToChangeFinal, url);
                        console.log('\x1b[1m\x1b[32m', 'Badge for', reportKey.key, 'updated with:', url);
                        returnValue = true;
                        fs.writeFileSync(readmeFile, readmeFileData, 'utf8');
                    }
                    else {
                        returnValue = false;
                    }
                });
                this.getBuildStatus();
            }
            else {
                console.log('\x1b[1m\x1b[31m', 'Error: ' + coveragePath + ' file not found. Is it at the default location?');
                console.log('\x1b[1m\x1b[33m', 'Do you have Jest installed? If so, is it properly configured? If you do, then run me');
                console.log('\x1b[1m\x1b[33m', "by passing args i.e. npm run jest-badges-readme --coverageDir='my-custom-coverage-folder'");
                returnValue = false;
            }
            return returnValue;
        };
        this.getBuildStatus = (path = badgeHashs.buildFile) => {
            let url = badgeHashs.getBuildUrl('Build', 'Passing', 'brightgreen');
            let returnValue = false;
            const pattern = `#${badgeHashs.hashes.build.value}#`;
            if (fs.existsSync(path)) {
                let readmeFileData = fs.readFileSync(readmeFile, 'utf8');
                const buildStatus = fs.readFileSync(path, 'utf8');
                if (buildStatus === 'ok') {
                    url = badgeHashs.getBuildUrl('Build', 'Passing', 'brightgreen');
                    returnValue = true;
                }
                else if (buildStatus === 'nok') {
                    url = badgeHashs.getBuildUrl('Build', 'Failing', 'brightred');
                    returnValue = false;
                }
                readmeFileData = readmeFileData.replace(pattern, url);
                fs.writeFileSync(readmeFile, readmeFileData, 'utf8');
                console.log('\x1b[1m\x1b[32m', 'Writing building status', buildStatus, 'with URL:', url);
            }
            else {
                returnValue = false;
            }
            return returnValue;
        };
        this.getCoveragePath = (path) => {
            let coveragePath = path;
            let argPath = '';
            const args = process.argv
                .filter((item) => {
                if (item.indexOf('coverage') >= 0) {
                    return item;
                }
            })
                .toString();
            if (args) {
                argPath = args.replace('--coverageDir=', '');
            }
            if (argPath && args.length > 0) {
                coveragePath = `${argPath}/coverage-summary.json`;
            }
            return coveragePath;
        };
        this.getBadge = (report, key) => {
            if (!(report && report.total && report.total[key])) {
                console.log('\x1b[1m\x1b[31m', 'Malformed coverage report for the key ' + key + '. Please run Jest again.');
                return '';
            }
            const coverage = report.total[key].pct;
            const color = this.getColor(coverage);
            return badgeHashs.getCoverageUrl(coverage, color);
        };
        this.getColor = (coverage) => {
            if (coverage < 80) {
                return 'red';
            }
            if (coverage < 90) {
                return 'yellow';
            }
            return 'brightgreen';
        };
    }
}
